@@ -41,15 +41,12 @@
 #include "volumes.h"
 #include "transaction.h"
 #include "utils.h"
-#include "version.h"
 
 static u64 index_cnt = 2;
 
 #define DEFAULT_MKFS_FEATURES	(BTRFS_FEATURE_INCOMPAT_EXTENDED_IREF \
 		| BTRFS_FEATURE_INCOMPAT_SKINNY_METADATA)
 
-#define DEFAULT_MKFS_LEAF_SIZE 16384
-
 struct directory_name_entry {
 	char *dir_name;
 	char *path;
@@ -275,29 +272,29 @@ static void print_usage(void)
 {
 	fprintf(stderr, "usage: mkfs.btrfs [options] dev [ dev ... ]\n");
 	fprintf(stderr, "options:\n");
-	fprintf(stderr, "\t -A --alloc-start the offset to start the FS\n");
-	fprintf(stderr, "\t -b --byte-count total number of bytes in the FS\n");
-	fprintf(stderr, "\t -d --data data profile, raid0, raid1, raid5, raid6, raid10, dup or single\n");
-	fprintf(stderr, "\t -f --force force overwrite of existing filesystem\n");
-	fprintf(stderr, "\t -l --leafsize size of btree leaves\n");
-	fprintf(stderr, "\t -L --label set a label\n");
-	fprintf(stderr, "\t -m --metadata metadata profile, values like data profile\n");
-	fprintf(stderr, "\t -M --mixed mix metadata and data together\n");
-	fprintf(stderr, "\t -n --nodesize size of btree nodes\n");
-	fprintf(stderr, "\t -s --sectorsize min block allocation (may not mountable by current kernel)\n");
-	fprintf(stderr, "\t -r --rootdir the source directory\n");
-	fprintf(stderr, "\t -K --nodiscard do not perform whole device TRIM\n");
-	fprintf(stderr, "\t -O --features comma separated list of filesystem features\n");
-	fprintf(stderr, "\t -U --uuid specify the filesystem UUID\n");
-	fprintf(stderr, "\t -V --version print the mkfs.btrfs version and exit\n");
-	fprintf(stderr, "%s\n", BTRFS_BUILD_VERSION);
+	fprintf(stderr, "\t-A|--alloc-start START  the offset to start the FS\n");
+	fprintf(stderr, "\t-b|--byte-count SIZE    total number of bytes in the FS\n");
+	fprintf(stderr, "\t-d|--data PROFILE       data profile, raid0, raid1, raid5, raid6, raid10, dup or single\n");
+	fprintf(stderr, "\t-f|--force              force overwrite of existing filesystem\n");
+	fprintf(stderr, "\t-l|--leafsize SIZE      deprecated, alias for nodesize\n");
+	fprintf(stderr, "\t-L|--label LABEL        set a label\n");
+	fprintf(stderr, "\t-m|--metadata PROFILE   metadata profile, values like data profile\n");
+	fprintf(stderr, "\t-M|--mixed              mix metadata and data together\n");
+	fprintf(stderr, "\t-n|--nodesize SIZE      size of btree nodes\n");
+	fprintf(stderr, "\t-s|--sectorsize SIZE    min block allocation (may not mountable by current kernel)\n");
+	fprintf(stderr, "\t-r|--rootdir DIR        the source directory\n");
+	fprintf(stderr, "\t-K|--nodiscard          do not perform whole device TRIM\n");
+	fprintf(stderr, "\t-O|--features LIST      comma separated list of filesystem features\n");
+	fprintf(stderr, "\t-U|--uuid UUID          specify the filesystem UUID\n");
+	fprintf(stderr, "\t-V|--version            print the mkfs.btrfs version and exit\n");
+	fprintf(stderr, "%s\n", PACKAGE_STRING);
 	exit(1);
 }
 
 static void print_version(void) __attribute__((noreturn));
 static void print_version(void)
 {
-	fprintf(stderr, "mkfs.btrfs, part of %s\n", BTRFS_BUILD_VERSION);
+	fprintf(stderr, "mkfs.btrfs, part of %s\n", PACKAGE_STRING);
 	exit(0);
 }
 
@@ -1058,27 +1055,6 @@ static int zero_output_file(int out_fd, u64 size, u32 sectorsize)
 	return ret;
 }
 
-static int check_leaf_or_node_size(u32 size, u32 sectorsize)
-{
-	if (size < sectorsize) {
-		fprintf(stderr,
-			"Illegal leafsize (or nodesize) %u (smaller than %u)\n",
-			size, sectorsize);
-		return -1;
-	} else if (size > BTRFS_MAX_METADATA_BLOCKSIZE) {
-		fprintf(stderr,
-			"Illegal leafsize (or nodesize) %u (larger than %u)\n",
-			size, BTRFS_MAX_METADATA_BLOCKSIZE);
-		return -1;
-	} else if (size & (sectorsize - 1)) {
-		fprintf(stderr,
-			"Illegal leafsize (or nodesize) %u (not align to %u)\n",
-			size, sectorsize);
-		return -1;
-	}
-	return 0;
-}
-
 static int is_ssd(const char *file)
 {
 	blkid_probe probe;
@@ -1231,7 +1207,8 @@ int main(int ac, char **av)
 	u64 alloc_start = 0;
 	u64 metadata_profile = 0;
 	u64 data_profile = 0;
-	u32 leafsize = max_t(u32, sysconf(_SC_PAGESIZE), DEFAULT_MKFS_LEAF_SIZE);
+	u32 leafsize = max_t(u32, sysconf(_SC_PAGESIZE),
+			BTRFS_MKFS_DEFAULT_NODE_SIZE);
 	u32 sectorsize = 4096;
 	u32 nodesize = leafsize;
 	u32 stripesize = 4096;
@@ -1360,9 +1337,9 @@ int main(int ac, char **av)
 		}
 	}
 	sectorsize = max(sectorsize, (u32)sysconf(_SC_PAGESIZE));
-	if (check_leaf_or_node_size(leafsize, sectorsize))
+	if (btrfs_check_node_or_leaf_size(leafsize, sectorsize))
 		exit(1);
-	if (check_leaf_or_node_size(nodesize, sectorsize))
+	if (btrfs_check_node_or_leaf_size(nodesize, sectorsize))
 		exit(1);
 	saved_optind = optind;
 	dev_cnt = ac - optind;
@@ -1441,7 +1418,7 @@ int main(int ac, char **av)
 		if (!leaf_forced) {
 			leafsize = best_leafsize;
 			nodesize = best_leafsize;
-			if (check_leaf_or_node_size(leafsize, sectorsize))
+			if (btrfs_check_node_or_leaf_size(leafsize, sectorsize))
 				exit(1);
 		}
 		if (leafsize != sectorsize) {
@@ -1489,8 +1466,8 @@ int main(int ac, char **av)
 	}
 
 	/* if we are here that means all devs are good to btrfsify */
-	printf("%s\n", BTRFS_BUILD_VERSION);
-	printf("See http://btrfs.wiki.kernel.org for more information.\n\n");
+	printf("%s\n", PACKAGE_STRING);
+	printf("See %s for more information.\n\n", PACKAGE_URL);
 
 	dev_cnt--;
 
